@@ -7,14 +7,14 @@ package AnyEvent::Redis::RipeRedis;
 
 use base qw( Exporter );
 
-our $VERSION = '1.43_01';
+our $VERSION = '1.45_01';
 
 use AnyEvent;
 use AnyEvent::Handle;
 use Encode qw( find_encoding is_utf8 );
 use Scalar::Util qw( looks_like_number weaken );
 use Digest::SHA qw( sha1_hex );
-use Carp qw( confess );
+use Carp qw( croak );
 
 BEGIN {
   our @EXPORT_OK = qw(
@@ -37,6 +37,7 @@ BEGIN {
     E_NO_AUTH
     E_WRONG_TYPE
     E_NO_REPLICAS
+    E_BUSY_KEY
   );
 
   our %EXPORT_TAGS = (
@@ -70,6 +71,7 @@ use constant {
   E_NO_AUTH                    => 19,
   E_WRONG_TYPE                 => 20,
   E_NO_REPLICAS                => 21,
+  E_BUSY_KEY                   => 22,
 
   # Operation status
   S_NEED_PERFORM => 1,
@@ -126,37 +128,30 @@ sub new {
 
   my $self = ref( $proto ) ? $proto : bless {}, $proto;
 
-  $self->{host}     = $params->{host} || D_HOST;
-  $self->{port}     = $params->{port} || D_PORT;
-  $self->{password} = $params->{password};
-  unless ( defined $params->{database} ) {
-    $params->{database} = D_DB_INDEX;
-  }
-  $self->{database} = $params->{database};
-  $self->encoding( $params->{encoding} );
-  $self->connection_timeout( $params->{connection_timeout} );
-  $self->read_timeout( $params->{read_timeout} );
-  unless ( exists $params->{reconnect} ) {
-    $params->{reconnect} = 1;
-  }
-  $self->{reconnect} = $params->{reconnect};
-
-  my $handle_params = $params->{handle_params};
-  if ( !defined $params->{handle_params} ) {
-    $handle_params = {};
-  }
-  foreach my $p_name ( qw( linger autocork ) ) {
-    if ( !exists $handle_params->{ $p_name } && defined $params->{ $p_name } ) {
-      $handle_params->{ $p_name } = $params->{ $p_name };
-    }
-  }
-  $self->{handle_params} = $handle_params;
-
+  $self->{host}             = $params->{host} || D_HOST;
+  $self->{port}             = $params->{port} || D_PORT;
+  $self->{password}         = $params->{password};
   $self->{on_connect}       = $params->{on_connect};
   $self->{on_disconnect}    = $params->{on_disconnect};
   $self->{on_connect_error} = $params->{on_connect_error};
+  $self->{database}         = defined $params->{database}
+      ? $params->{database} : D_DB_INDEX;
+  $self->{reconnect}        = exists $params->{reconnect}
+      ? $params->{reconnect} : 1;
+
+  $self->encoding( $params->{encoding} );
+  $self->connection_timeout( $params->{connection_timeout} );
+  $self->read_timeout( $params->{read_timeout} );
   $self->on_error( $params->{on_error} );
 
+  my $hdl_params = $params->{handle_params} || {};
+  foreach my $name ( qw( linger autocork ) ) {
+    if ( !exists $hdl_params->{$name} && defined $params->{$name} ) {
+      $hdl_params->{$name} = $params->{$name};
+    }
+  }
+  $self->{handle_params} = $hdl_params;
+
   $self->{_handle}         = undef;
   $self->{_connected}      = 0;
   $self->{_lazy_conn_st}   = $params->{lazy};
@@ -180,11 +175,9 @@ sub new {
 ####
 sub multi {
   my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'multi', [ @_ ] );
+  my $cmd  = $self->_prepare_cmd( 'multi', [ @_ ] );
 
   $self->{_multi_lock} = 1;
-
   $self->_execute_cmd( $cmd );
 
   return;
@@ -193,65 +186,18 @@ sub multi {
 ####
 sub exec {
   my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'exec', [ @_ ] );
+  my $cmd  = $self->_prepare_cmd( 'exec', [ @_ ] );
 
   $self->{_multi_lock} = 0;
-
   $self->_execute_cmd( $cmd );
 
   return;
 }
 
-####
-sub subscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'subscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub psubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'psubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub unsubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'unsubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
-####
-sub punsubscribe {
-  my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'punsubscribe', [ @_ ] );
-
-  $self->_subunsub( $cmd );
-
-  return;
-}
-
 ####
 sub eval_cached {
   my $self = shift;
-
-  my $cmd = $self->_prepare_cmd( 'evalsha', [ @_ ] );
+  my $cmd  = $self->_prepare_cmd( 'evalsha', [ @_ ] );
 
   $cmd->{script} = $cmd->{args}[0];
   unless ( exists $EVAL_CACHE{ $cmd->{script} } ) {
@@ -282,8 +228,9 @@ sub encoding {
 
     if ( defined $enc ) {
       $self->{encoding} = find_encoding( $enc );
+
       unless ( defined $self->{encoding} ) {
-        confess "Encoding '$enc' not found";
+        croak "Encoding '$enc' not found";
       }
     }
     else {
@@ -323,43 +270,49 @@ sub selected_database {
   return $self->{database};
 }
 
-# Create more accessors
+# Generate additional methods and accessors
 {
   no strict 'refs';
 
-  foreach my $attr_pref ( qw( connection read ) ) {
-    my $attr_name = $attr_pref . '_timeout';
+  foreach my $kwd ( keys %SUBUNSUB_CMDS ) {
+    *{$kwd} = sub {
+      my $self = shift;
+      my $cmd  = $self->_prepare_cmd( $kwd, [ @_ ] );
 
-    *{$attr_name} = sub {
+      $self->_subunsub( $cmd );
+
+      return;
+    },
+  }
+
+  foreach my $name_pref ( qw( connection read ) ) {
+    my $name = $name_pref . '_timeout';
+
+    *{$name} = sub {
       my $self = shift;
 
       if ( @_ ) {
         my $timeout = shift;
 
-        if (
-          defined $timeout
-            && ( !looks_like_number( $timeout ) || $timeout < 0 )
-            ) {
-          confess ucfirst( $attr_pref ) . ' timeout must be a positive number';
+        if ( defined $timeout && ( !looks_like_number( $timeout ) || $timeout < 0 ) ) {
+          croak ucfirst( $name_pref ) . ' timeout must be a positive number';
         }
-        $self->{ $attr_name } = $timeout;
+        $self->{$name} = $timeout;
       }
 
-      return $self->{ $attr_name };
+      return $self->{$name};
     }
   }
 
-  foreach my $attr_name (
-    qw( reconnect on_connect on_disconnect on_connect_error )
-      ) {
-    *{$attr_name} = sub {
+  foreach my $name ( qw( reconnect on_connect on_disconnect on_connect_error ) ) {
+    *{$name} = sub {
       my $self = shift;
 
       if ( @_ ) {
-        $self->{ $attr_name } = shift;
+        $self->{$name} = shift;
       }
 
-      return $self->{ $attr_name };
+      return $self->{$name};
     }
   }
 }
@@ -610,16 +563,16 @@ sub _prepare_cmd {
 
   my $cmd;
   if ( ref( $args->[-1] ) eq 'HASH' ) {
-    $cmd = pop( @{$args} );
+    $cmd = pop @{$args};
   }
   else {
     $cmd = {};
     if ( ref( $args->[-1] ) eq 'CODE' ) {
-      if ( exists $SUB_CMDS{ $kwd } ) {
-        $cmd->{on_message} = pop( @{$args} );
+      if ( exists $SUB_CMDS{$kwd} ) {
+        $cmd->{on_message} = pop @{$args};
       }
       else {
-        $cmd->{on_reply} = pop( @{$args} );
+        $cmd->{on_reply} = pop @{$args};
       }
     }
   }
@@ -635,7 +588,7 @@ sub _subunsub {
   my $cmd  = shift;
 
   if ( exists $SUB_CMDS{ $cmd->{kwd} } && !defined $cmd->{on_message} ) {
-    confess "'on_message' callback must be specified";
+    croak "'on_message' callback must be specified";
   }
 
   if ( $self->{_multi_lock} ) {
@@ -651,13 +604,13 @@ sub _subunsub {
     return;
   }
 
-  $cmd->{repls_left} = scalar( @{$cmd->{args}} );
+  $cmd->{repls_left} = scalar @{ $cmd->{args} };
 
   if ( defined $cmd->{on_done} ) {
     my $on_done = $cmd->{on_done};
 
     $cmd->{on_done} = sub {
-      $on_done->( @{$_[0]} );
+      $on_done->( @{ $_[0] } );
     }
   }
 
@@ -694,9 +647,8 @@ sub _execute_cmd {
     else {
       AE::postpone(
         sub {
-          $self->_process_cmd_error( $cmd,
-              "Operation '$cmd->{kwd}' aborted: No connection to the server.",
-              E_NO_CONN );
+          $self->_process_cmd_error( $cmd, "Operation '$cmd->{kwd}' aborted:"
+              . ' No connection to the server.', E_NO_CONN );
         }
       );
 
@@ -736,6 +688,7 @@ sub _push_write {
     $handle->rtimeout( $self->{read_timeout} );
   }
   push( @{$self->{_process_queue}}, $cmd );
+
   $handle->push_write( $cmd_str );
 
   return;
@@ -807,9 +760,10 @@ sub _select_db {
 sub _flush_input_queue {
   my $self = shift;
 
-  $self->{_tmp_queue} = $self->{_input_queue};
+  $self->{_tmp_queue}   = $self->{_input_queue};
   $self->{_input_queue} = [];
-  while ( my $cmd = shift( @{$self->{_tmp_queue}} ) ) {
+
+  while ( my $cmd = shift @{ $self->{_tmp_queue} } ) {
     $self->_push_write( $cmd );
   }
 
@@ -818,10 +772,12 @@ sub _flush_input_queue {
 
 ####
 sub _process_reply {
-  my $self = shift;
+  my $self     = shift;
+  my $data     = shift;
+  my $err_code = shift;
 
-  if ( defined $_[1] ) {
-    my $cmd = shift( @{$self->{_process_queue}} );
+  if ( defined $err_code ) {
+    my $cmd = shift @{$self->{_process_queue}};
 
     unless ( defined $cmd ) {
       $self->_disconnect( "Don't known how process error."
@@ -830,26 +786,21 @@ sub _process_reply {
       return;
     }
 
-    $self->_process_cmd_error( $cmd, ref( $_[0] )
-        ? ( "Operation '$cmd->{kwd}' completed with errors.", @_[ 1, 0 ] )
-        : @_ );
+    $self->_process_cmd_error( $cmd, ref( $data )
+        ? ( "Operation '$cmd->{kwd}' completed with errors.", $err_code, $data )
+        : $data, $err_code );
   }
-  elsif (
-    $self->{_subs_num} > 0 && ref( $_[0] )
-      && exists $MSG_TYPES{ $_[0][0] }
-      ) {
-    my $msg    = $_[0];
-    my $on_msg = $self->{_subs}{ $msg->[1] };
+  elsif ( $self->{_subs_num} > 0 && ref( $data ) && exists $MSG_TYPES{ $data->[0] } ) {
+    my $on_msg = $self->{_subs}{ $data->[1] };
 
     unless ( defined $on_msg ) {
       $self->_disconnect( "Don't known how process published message."
-          . " Unknown channel or pattern '$msg->[1]'.", E_UNEXPECTED_DATA );
+          . " Unknown channel or pattern '$data->[1]'.", E_UNEXPECTED_DATA );
 
       return;
     }
 
-    $on_msg->( $msg->[0] eq 'pmessage' ? @{$msg}[ 2, 3, 1 ]
-        : @{$msg}[ 1, 2 ] );
+    $on_msg->( $data->[0] eq 'pmessage' ? @{$data}[ 2, 3, 1 ] : @{$data}[ 1, 2 ] );
   }
   else {
     my $cmd = $self->{_process_queue}[0];
@@ -862,10 +813,9 @@ sub _process_reply {
     }
 
     if ( !defined $cmd->{repls_left} || --$cmd->{repls_left} <= 0 ) {
-      shift( @{$self->{_process_queue}} );
+      shift @{$self->{_process_queue}};
     }
-
-    $self->_process_cmd_success( $cmd, $_[0] );
+    $self->_process_cmd_success( $cmd, $data );
   }
 
   return;
@@ -878,7 +828,7 @@ sub _process_cmd_error {
 
   if ( $_[1] == E_NO_SCRIPT && defined $cmd->{script} ) {
     $cmd->{kwd}     = 'eval';
-    $cmd->{args}[0] = $cmd->{script};
+    $cmd->{args}[0] = delete $cmd->{script};
 
     $self->_push_write( $cmd );
 
@@ -907,14 +857,14 @@ sub _process_cmd_success {
   if ( exists $NEED_SPECPROC{ $cmd->{kwd} } ) {
     my $kwd = $cmd->{kwd};
 
-    if ( exists $SUBUNSUB_CMDS{ $kwd } ) {
-      shift( @{$data} );
+    if ( exists $SUBUNSUB_CMDS{$kwd} ) {
+      shift @{$data};
 
-      if ( exists $SUB_CMDS{ $kwd } ) {
+      if ( exists $SUB_CMDS{$kwd} ) {
         $self->{_subs}{ $data->[0] } = $cmd->{on_message};
       }
       else { # unsubscribe or punsubscribe
-        delete( $self->{_subs}{ $data->[0] } );
+        delete $self->{_subs}{ $data->[0] };
       }
 
       $self->{_subs_num} = $data->[1];
@@ -942,8 +892,10 @@ sub _process_cmd_success {
 
 ####
 sub _parse_info {
-  return { map { split( m/:/, $_, 2 ) } grep( m/^[^#]/ ,
-      split( EOL, $_[1] ) ) };
+  return {
+    map { split( m/:/, $_, 2 ) }
+    grep { m/^[^#]/ } split( EOL, $_[1] )
+  };
 }
 
 ####
@@ -1002,8 +954,8 @@ sub _abort_all {
     }
 
     foreach my $cmd ( @unfin_cmds ) {
-      $self->_process_cmd_error( $cmd,
-          "Operation '$cmd->{kwd}' aborted: $err_msg", $err_code );
+      $self->_process_cmd_error( $cmd, "Operation '$cmd->{kwd}' aborted: "
+          . $err_msg, $err_code );
     }
   }
 
@@ -1019,8 +971,7 @@ sub AUTOLOAD {
 
   my $sub = sub {
     my $self = shift;
-
-    my $cmd = $self->_prepare_cmd( $kwd, [ @_ ] );
+    my $cmd  = $self->_prepare_cmd( $kwd, [ @_ ] );
 
     $self->_execute_cmd( $cmd );
 
@@ -1111,7 +1062,7 @@ feature
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1152,9 +1103,11 @@ feature
 
   $redis->quit(
     sub {
-      if ( defined $_[1] ) {
-        my $err_msg  = $_[1];
-        my $err_code = $_[2];
+      shift;
+
+      if ( @_ ) {
+        my $err_msg  = shift;
+        my $err_code = shift;
 
         warn "[$err_code] $err_msg\n";
       }
@@ -1359,7 +1312,7 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1376,7 +1329,7 @@ The full list of the Redis commands can be found here: L<http://redis.io/command
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1480,19 +1433,25 @@ error message and C<code()> to get error code.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $oprn_err_msg  = $chunk->message();
-            my $oprn_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $oprn_err_msg  = $chunk->message();
+              my $oprn_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
+
+          return;
         }
 
+        # error handling...
+
         return;
       }
 
@@ -1516,14 +1475,18 @@ error message and C<code()> to get error code.
         my $err_code = shift;
         my $data     = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $oprn_err_msg  = $chunk->message();
-            my $oprn_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $oprn_err_msg  = $chunk->message();
+              my $oprn_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
+
+        # error handling...
       },
     }
   );
@@ -1560,7 +1523,7 @@ and C<PUNSUBSCRIBE> commands.
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1650,7 +1613,7 @@ Subscribes the client to the given patterns.
     { on_reply => sub {
         my $data = shift;
 
-        if ( defined $_[0] ) {
+        if ( @_ ) {
           my $err_msg  = shift;
           my $err_code = shift;
 
@@ -1752,7 +1715,7 @@ channel will be sent to the client.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1812,7 +1775,7 @@ pattern will be sent to the client.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1892,7 +1855,7 @@ instead.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
@@ -1920,19 +1883,23 @@ command.
     sub {
       my $data = shift;
 
-      if ( defined $_[0] ) {
+      if ( @_ ) {
         my $err_msg  = shift;
         my $err_code = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $nested_err_msg  = $chunk->message();
-            my $nested_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $nested_err_msg  = $chunk->message();
+              my $nested_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
 
+        # error handling...
+
         return;
       }
 
@@ -1952,14 +1919,18 @@ command.
         my $err_code = shift;
         my $data     = shift;
 
-        foreach my $chunk ( @{$data} ) {
-          if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
-            my $nested_err_msg  = $chunk->message();
-            my $nested_err_code = $chunk->code();
+        if ( defined $data ) {
+          foreach my $chunk ( @{$data} ) {
+            if ( ref( $chunk ) eq 'AnyEvent::Redis::RipeRedis::Error' ) {
+              my $nested_err_msg  = $chunk->message();
+              my $nested_err_code = $chunk->code();
 
-            # error handling...
+              # error handling...
+            }
           }
         }
+
+        # error handling...
       }
     }
   );
@@ -2081,6 +2052,10 @@ Operation against a key holding the wrong kind of value.
 
 Not enough good slaves to write.
 
+=item E_BUSY_KEY
+
+Target key name already exists.
+
 =back
 
 =head1 DISCONNECTION
